
    /*    this is a mixup of classes Vector3D from the following libraries:
     *
     *    http://www.shiffman.net/teaching/nature/library/
     *    //import noc.*;
     *
     *    http://www.cs.princeton.edu/~traer/physics/
     *    //import traer.physics.*;
     */


public class Vector3D
{

    public Vector3D(float X, float Y, float Z)
    {
        x = X;
        y = Y;
        z = Z;
    }

    public Vector3D()
    {
        x = 0.0F;
        y = 0.0F;
        z = 0.0F;
    }

    public Vector3D(Vector3D p)
    {
        x = p.x;
        y = p.y;
        z = p.z;
    }

    public final float z()
    {
        return z;
    }

    public final float y()
    {
        return y;
    }

    public final float x()
    {
        return x;
    }

    public final void setX(float X)
    {
        x = X;
    }

    public final void setY(float Y)
    {
        y = Y;
    }

    public final void setZ(float Z)
    {
        z = Z;
    }

    public final void set(float X, float Y, float Z)
    {
        x = X;
        y = Y;
        z = Z;
    }

    public final void set(Vector3D p)
    {
        x = p.x;
        y = p.y;
        z = p.z;
    }

    public final Vector3D add(Vector3D p)
    {
        x += p.x;
        y += p.y;
        z += p.z;
        return this;
    }
    
    /**
     * Add two vectors
     * @param      v1 a vector
     * @param v2 another vector   
     * @return a new vector that is the sum of v1 and v2  
     */   
    public static Vector3D add(Vector3D v1, Vector3D v2) {
        Vector3D v = new Vector3D(v1.x + v2.x,v1.y + v2.y, v1.z + v2.z);
        return v;
    }
    
    /**
     * Subtract one vector from another
     * @param      v1 a vector
     * @param v2 another vector   
     * @return a new vector that is v1 - v2 
     */    
    public static Vector3D subtract(Vector3D v1, Vector3D v2) {
        Vector3D v = new Vector3D(v1.x - v2.x,v1.y - v2.y,v1.z - v2.z);
        return v;
    }

    public final void subtract(Vector3D p)
    {
        x -= p.x;
        y -= p.y;
        z -= p.z;
    }

    public final void add(float a, float b, float c)
    {
        x += a;
        y += b;
        z += c;
    }

    public final Vector3D plus(Vector3D p)
    {
        return new Vector3D(x + p.x, y + p.y, z + p.z);
    }

    public final Vector3D times(float f)
    {
        return new Vector3D(x * f, y * f, z * f);
    }

    public final Vector3D over(float f)
    {
        return new Vector3D(x / f, y / f, z / f);
    }

    public final Vector3D minus(Vector3D p)
    {
        return new Vector3D(x - p.x, y - p.y, z - p.z);
    }

    public final Vector3D multiplyBy(float f)
    {
        x *= f;
        y *= f;
        z *= f;
        return this;
    }

    public final float distanceTo(Vector3D p)
    {
        float dx = x - p.x;
        float dy = y - p.y;
        float dz = z - p.z;
        return fastSqrt(dx * dx + dy * dy + dz * dz);
    }

    public final float distanceTo(float x, float y, float z)
    {
        float dx = this.x - x;
        float dy = this.y - y;
        float dz = this.z - z;
        return fastSqrt(dx * dx + dy * dy + dz * dz);
    }

    public final float dot(Vector3D p)
    {
        return x * p.x + y * p.y + z * p.z;
    }

    public final float length()
    {
        return (float)Math.sqrt(x * x + y * y + z * z);
    }

    public final Vector3D unit()
    {
        float l = length();
        return l != 0.0F ? over(l) : new Vector3D();
    }

    public final void clear()
    {
        x = 0.0F;
        y = 0.0F;
        z = 0.0F;
    }
    
    /**
     * Copy the vector
     * @return      a copy of the vector   
     */
    public Vector3D copy() {
        return new Vector3D(x,y,z);
    }
    
    /**
     * Copy the vector
     * @param      v the vector to be copied   
     * @return      a copy of the vector   
     */
    public static Vector3D copy(Vector3D v) {
        return new Vector3D(v.x, v.y,v.z);
    }
    
    /**
     * Calculate the angle of rotation for this vector (only 2D vectors)
     * @return the angle of rotation
     */    
    public float heading2D() {
        float angle = (float) Math.atan2(-y, x);
        return -1*angle;
    }
    
    /**
     * Normalize the vector to length 1 (make it a unit vector)
     */     
    public void normalize() {
        float m = magnitude();
        if (m > 0) {
            div(m);
        }
    }
    
    /**
     * Divide this vector by a scalar
     * @param      n the value to divide by 
     */     
    public void div(float n) {
        x /= n;
        y /= n;
        z /= n;
    }
    
    /**
     * Calculate the magnitude (length) of the vector
     * @return      the magnitude of the vector    
     */
    public float magnitude() {
        return fastSqrt(x*x + y*y + z*z);
        //return (float) Math.sqrt(x*x + y*y + z*z);
    }
    
    /**
     * Limit the magnitude of this vector
     * @param max the maximum length to limit this vector
     */     
    public void limit(float max) {
        if (magnitude() > max) {
            normalize();
            multiplyBy(max);
        }
    }
    
    /**
     * Set x,y, and z coordinates.
     *     
     *  @param  x_ the x coordinate.
     *  @param  y_ the y coordinate.
     *  @param  z_ the z coordinate.
     */
    public void setXYZ(float x_, float y_, float z_) {
        x = x_;
        y = y_;
        z = z_;
    }
    
    /**
     * Set x,y, and z coordinates from a Vector3D object.
     *     
     *  @param  v the Vector3D object to be copied
     */
    public void setXYZ(Vector3D v) {
        x = v.x;
        y = v.y;
        z = v.z;
    }

    public final String toString()
    {
        return new String("(" + x + ", " + y + ", " + z + ")");
    }

    public final Vector3D cross(Vector3D p)
    {
        return new Vector3D(y * p.z - z * p.y, x * p.z - z * p.x, x * p.y - y * p.x);
    }

    public static float fastInverseSqrt(float x)
    {
        float half = 0.5F * x;
        int i = Float.floatToIntBits(x);
        i = Vector3D.LOMONT_MAGIC_NUMBER - (i >> 1);
        x = Float.intBitsToFloat(i);
        return x * (1.5F - half * x * x);
    }
    
    public static float fastSqrt (float val)
    {
        return 1.0f/fastInverseSqrt( val );
    }
    
    public static float angleBetween(Vector3D v1, Vector3D v2)
    {
        float dot = v1.dot(v2);
        float theta = (float) Math.acos(dot / (v1.magnitude() * v2.magnitude()));
        return theta;
    }

    public float x;
    public float y;
    public float z;
    private static final int LOMONT_MAGIC_NUMBER = 0x5f375a86;
}
